summary refs log tree commit diff
path: root/synapse/rest
diff options
context:
space:
mode:
authorDavid Baker <dbkr@matrix.org>2014-09-23 17:16:13 +0100
committerDavid Baker <dbkr@matrix.org>2014-09-23 17:16:13 +0100
commit14ed6799d72c7807467456808aa08a6f376ebe14 (patch)
tree19df0878a63688b992a437240de151dfd1ab7333 /synapse/rest
parentPartial fix of SYWEB-28: If members do not have last_active_ago, compare thei... (diff)
downloadsynapse-14ed6799d72c7807467456808aa08a6f376ebe14.tar.xz
Add support for TURN servers as per the TURN REST API (http://tools.ietf.org/html/draft-uberti-behave-turn-rest-00)
Diffstat (limited to 'synapse/rest')
-rw-r--r--synapse/rest/__init__.py3
-rw-r--r--synapse/rest/voip.py59
2 files changed, 61 insertions, 1 deletions
diff --git a/synapse/rest/voip.py b/synapse/rest/voip.py
new file mode 100644
index 0000000000..cba9b27e3b
--- /dev/null
+++ b/synapse/rest/voip.py
@@ -0,0 +1,59 @@
+# -*- coding: utf-8 -*-
+# Copyright 2014 OpenMarket Ltd
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+from twisted.internet import defer
+
+from base import RestServlet, client_path_pattern
+
+from syutil.base64util import encode_base64
+
+import hmac
+import hashlib
+
+
+class VoipRestServlet(RestServlet):
+    PATTERN = client_path_pattern("/voip/turnuris$")
+
+    @defer.inlineCallbacks
+    def on_GET(self, request):
+        auth_user = yield self.auth.get_user_by_req(request)
+
+        turnUri = self.hs.config.voip.turn_uri
+        turnSecret = self.hs.config.voip.turn_shared_secret
+        userLifetime = self.hs.config.voip.turn_user_lifetime
+        if not turnUri or not turnSecret or not userLifetime:
+            defer.returnValue( (200, {"uris": []}) )
+
+        expiry = self.hs.get_clock().time_msec() + userLifetime
+        username = "%d:%s" % (expiry, auth_user.to_string())
+         
+        mac = hmac.new(turnSecret, msg=username, digestmod=hashlib.sha1)
+        password = encode_base64(mac.digest())
+
+        defer.returnValue( (200, {
+            'username': username,
+            'password': password,
+            'ttl': userLifetime / 1000,
+            'uris': [
+                turnUri,
+            ]
+        }) )
+
+    def on_OPTIONS(self, request):
+        return (200, {})
+
+
+def register_servlets(hs, http_server):
+    VoipRestServlet(hs).register(http_server)
diff --git a/synapse/rest/__init__.py b/synapse/rest/__init__.py
index ed785cfbd5..3b9aa59733 100644
--- a/synapse/rest/__init__.py
+++ b/synapse/rest/__init__.py
@@ -15,7 +15,7 @@
 
 
 from . import (
     room, events, register, login, profile, presence, initial_sync, directory
 )
 
 
@@ -42,3 +42,4 @@ class RestServletFactory(object):
         presence.register_servlets(hs, client_resource)
         initial_sync.register_servlets(hs, client_resource)
         directory.register_servlets(hs, client_resource)
+        voip.register_servlets(hs, client_resource)
@@ -15,7 +15,7 @@
 
 from . import (
-    room, events, register, login, profile, presence, initial_sync, directory
+    room, events, register, login, profile, presence, initial_sync, directory, voip
 )
 